*** Program: Interactions between effect modifier (category) and exposure(category) in a Cox survival model

*** Last update: Oct 9, 2014 by Yingying Sang (Johns Hopkins Bloomberg School of Public Health, ysang1@jhu.edu)
******************************************************************************************************************

program cat_intx
version 11
syntax varlist(min=2 max=3) [if] [in], l1(integer) [l2(integer 0) r1(integer 0) r2(integer 0) fu(varname) outcome(varname) ///
       lm(integer 1) covar(varlist fv) name(string) tbl resh]
preserve
marksample touse
qui keep if `touse'
if _N==0 {
   display "No observations"
   exit
   }
tempvar modcat indvarcat
tempname B
tokenize "`varlist'"
local modifier `1'
local cat1 `2'
if "`3'"!="" local cat2 `3'
if "`fu'"!="" stset `fu', failure(`outcome')
else {
   capture list _st
   if _rc>0 {
      display "Data not stset!"
	  exit
	  }
   }
count if _d==1
if r(N)<10 {
   display "Not enough events"
   exit
   }
if "`cat2'"!="" gen `indvarcat'=`cat1'*(`l2'+1)+`cat2'
else gen `indvarcat'=`cat1'
local modref=`r1'*(`l2'+1)+`r2'
capture noisily stcox i.`modifier'#ib`modref'.`indvarcat' `covar', nolog
if _rc!=0 exit
keep if e(sample)
forvalues k=0/`lm' {
   count if `modifier'==`k' & _d==1
   if r(N)<10 local ev`k' "Not enough events"
   }
tab2 `indvarcat' `modifier'
tab2 `indvarcat' `modifier' if _d
contract _d `modifier' `indvarcat'
reshape wide _freq, i(`modifier' `indvarcat') j(_d)
rename _freq0 participant
rename _freq1 event
gen `modcat'=`modifier'*(`l1'+1)*(`l2'+1)+`indvarcat'
while _N<`=(`lm'+1)*(`l1'+1)*(`l2'+1)' {
   set obs `=_N+1'
   forvalues i=1/`=_N-1' {
      if `modcat'[`i']!=`i'-1 {
	     qui replace `modcat'=`i'-1 in `=_N'
		 sort `modcat'
		 continue, break
		 }
	  }
   qui replace `modcat'=_N-1 if `modcat'==.
   }
qui replace participant=0 if participant==.
qui replace event=0 if event==.
qui replace participant=participant+event
qui replace `modifier'=int(`modcat'/((`l1'+1)*(`l2'+1)))
qui replace `indvarcat'=mod(`modcat',(`l1'+1)*(`l2'+1))
keep `modifier' `indvarcat' participant event
reshape wide participant event, i(`indvarcat') j(`modifier')		 
gen `modifier'=_n-1
forvalues k=0/`lm' {
   qui gen inr`k'=event`k'==0
   if "`ev`k''"!="" qui replace inr`k'=1
   }
qui replace inr0=1 if inr0[`=`modref'+1']==1	 
forvalues k=1/`lm' {
   gen inr`=`k'+`lm''=inr`k' | inr0[`=`modref'+1']
   qui replace inr`k'=1 if inr`k'[`=`modref'+1']==1	 
   gen inr`=`k'+2*`lm''=inr0 | inr`k'
   }
forvalues q=0/`=3*`lm'' {
     qui gen estimate`q'=.
	 qui gen stderr`q'=.
	 }
forvalues i=0/`=(`l1'+1)*(`l2'+1)-1' {
   capture lincom 0.`modifier'#`i'.`indvarcat'
   if _rc==0 {
      qui replace estimate0=r(estimate) in `=`i'+1'
	  qui replace stderr0=r(se) in `=`i'+1'
	  }
   forvalues k=1/`lm' {
      capture lincom `k'.`modifier'#`i'.`indvarcat'-`k'.`modifier'#`modref'.`indvarcat'
	  if _rc==0 {
		 qui replace estimate`k'=r(estimate) in `=`i'+1'
		 qui replace stderr`k'=r(se) in `=`i'+1'
		 }
      capture lincom `k'.`modifier'#`i'.`indvarcat'
	  if _rc==0 {
	     qui replace estimate`=`k'+`lm''=r(estimate) in `=`i'+1'
		 qui replace stderr`=`k'+`lm''=r(se) in `=`i'+1'
		 }
	  capture lincom `k'.`modifier'#`i'.`indvarcat'-0.`modifier'#`i'.`indvarcat'-`k'.`modifier'#`modref'.`indvarcat'
	  if _rc==0 {
		 qui replace estimate`=`k'+2*`lm''=r(estimate) in `=`i'+1'
		 qui replace stderr`=`k'+2*`lm''=r(se) in `=`i'+1'
		 }
	  }
   }  
forvalues q=0/`=3*`lm'' {
   qui replace inr`q'=2 if abs(estimate`q')>1.96*stderr`q' & inr`q'!=1
   gen hr`q'=exp(estimate`q')
   gen lci`q'=exp(estimate`q'-1.96*stderr`q')
   gen uci`q'=exp(estimate`q'+1.96*stderr`q')
   }
gen `cat1'=int(`indvarcat'/(`l2'+1))
if "`cat2'"!="" gen `cat2'=mod(`indvarcat',(`l2'+1))
drop `indvarcat' `modifier' participant* event*
save "`name'-estimate.dta", replace
if "`tbl'"!="" {
   forvalues k=0/`lm' {
	  if `k'>0 gen p`k'="*" if inr`=2*`lm'+`k''==2
	  else gen p`k'=""
      gen ci`k'_0=string(hr`k',"%8.2f")+p`k' if inr`k'!=1
	  gen ci`k'_1="("+string(lci`k',"%8.2f")+", "+string(uci`k',"%8.2f")+")" if inr`k'!=1
	  local civar "`civar' ci`k'_"
	  local civars "`civars' ci`k'_*"
	  }
   keep `cat1' `cat2' ci*
   reshape long `civar', i(`cat1' `cat2') j(eci)
   if "`resh'"!="" {
	  reshape wide `civar', i(`cat2' eci) j(`cat1')
      local cat1 `cat2'
	  }
   outsheet `cat1' `civars' using "`name'-estimate.csv", replace comma
   }
end
